@@ -54,14 +54,16 @@ class OSDMenu(object):
         self.menu_list = []
         self.menu_selection = 0
         self.menu_tmp = None
+        self.mouse_back = False
         self.original_osd_color = playerManager._player.osd_back_color
         self.original_osd_size = playerManager._player.osd_font_size
 
         self.profile_menu = None
+        self.profile_manager = None
         if settings.shader_pack_enable:
             try:
-                profile_manager = VideoProfileManager(self, playerManager)
-                self.profile_menu = profile_manager.menu_action
+                self.profile_manager = VideoProfileManager(self, playerManager)
+                self.profile_menu = self.profile_manager.menu_action
             except Exception:
                 log.error("Could not load profile manager.", exc_info=True)
         
@@ -82,15 +84,28 @@ class OSDMenu(object):
         items = self.menu_list
         selected_item = self.menu_selection
 
-        menu_text = "{0}".format(self.menu_title)
+        if self.mouse_back:
+            menu_text = "(<--) {0}".format(self.menu_title)
+        else:
+            menu_text = self.menu_title
         for i, item in enumerate(items):
             fmt = "\n   {0}"
-            if i == selected_item:
+            if i == selected_item and not self.mouse_back:
                 fmt = "\n   **{0}**"
             menu_text += fmt.format(item[0])
 
         self.playerManager._player.show_text(menu_text,2**30,1)
 
+    def mouse_select(self, idx):
+        if idx < 0 or idx > len(self.menu_list):
+            return
+        if idx == 0:
+            self.mouse_back = True
+        else:
+            self.mouse_back = False
+            self.menu_selection = idx - 1
+        self.refresh_menu()
+
     def show_menu(self):
         self.is_menu_shown = True
         player = self.playerManager._player
@@ -100,8 +115,11 @@ class OSDMenu(object):
         if hasattr(player, 'osc'):
             player.osc = False
         
+        player.command("script-message", "shim-menu-enable", "True")
+
         self.menu_title = "Main Menu"
         self.menu_selection = 0
+        self.mouse_back = False
 
         if self.playerManager._media_item and not player.playback_abort:
             self.menu_list = [
@@ -153,6 +171,8 @@ class OSDMenu(object):
             if hasattr(player, 'osc'):
                 player.osc = settings.enable_osc
 
+            player.command("script-message", "shim-menu-enable", "False")
+
             if player.playback_abort:
                 player.play("")
             else:
@@ -183,9 +203,13 @@ class OSDMenu(object):
                 else:
                     self.menu_title, self.menu_list, self.menu_selection = self.menu_stack.get_nowait()
             elif action == "ok":
-                self.menu_list[self.menu_selection][1]()
+                if self.mouse_back:
+                    self.menu_action("back")
+                else:
+                    self.menu_list[self.menu_selection][1]()
             elif action == "home":
                 self.show_menu()
+            self.mouse_back = False
             self.refresh_menu()
 
     def change_audio_menu_handle(self):
@@ -372,7 +396,7 @@ class OSDMenu(object):
         ])
 
     def preferences_menu(self):
-        self.put_menu("Preferences", [
+        options = [
             self.get_settings_toggle("Always Skip Intros", "skip_intro_always"),
             self.get_settings_toggle("Ask to Skip Intros", "skip_intro_prompt"),
             ("Transcode Quality: {0:0.1f} Mbps".format(settings.transcode_kbps/1000), self.transcode_settings_menu),
@@ -386,7 +410,38 @@ class OSDMenu(object):
             self.get_settings_toggle("Limit Direct Play", "direct_limit"),
             self.get_settings_toggle("Auto Fullscreen", "fullscreen"),
             self.get_settings_toggle("Media Key Seek", "media_key_seek"),
-        ])
+        ]
+
+        if self.profile_menu is not None:
+            options.append(
+                (
+                    "Video Profile Subtype: {0}".format(
+                        settings.shader_pack_subtype
+                    ),
+                    self.shader_pack_subtype_menu,
+                ),
+            )
+
+        self.put_menu("Preferences", options)
+
+    def shader_pack_subtype_menu(self):
+        self.put_menu(
+            "Video Profile Subtype",
+            [
+                (option, self.shader_pack_subtype_handle)
+                for option in self.profile_manager.profile_subtypes
+            ],
+        )
+
+    def shader_pack_subtype_handle(self):
+        option_value = self.menu_list[self.menu_selection][0]
+        settings.shader_pack_subtype = option_value
+        settings.save()
+
+        # Need to re-render preferences menu.
+        for i in range(2):
+            self.menu_action("back")
+        self.video_preferences_menu()
 
     def unwatched_menu_handle(self):
         self.playerManager.put_task(self.playerManager.unwatched_quit)
